/*
 * inventory.js - defines Inventory object which maintains an up to date 
 *                inventory based on the vmInventory file.  The Inventory 
 *                object keeps itself up to date when the vmInventory file
 *                changes on disk.
 *
 * Copyright (c) 2014 Sam Khavari
 * licensed under the MIT license.
 *
 */
var fs          = require('fs'),
    events      = require('events'),
    chokidar    = require('chokidar'),
    debug       = require('debug')("vmshell:inventory"),
    properties  = require('properties');

// - --------------------------------------------
// - the mac filename of the vmInventory file used by fusion
// - 
var filename = "/Users/" + process.env.USER + '/Library/Application Support/VMware Fusion/vmInventory';


function Inventory() {
    events.EventEmitter.call(this);
}

Inventory.prototype.__proto__ = events.EventEmitter.prototype;

// - path to vmInventory (mac only for now)
Inventory.filename = "/Users/" + process.env.USER + '/Library/Application Support/VMware Fusion/vmInventory';

// - list of vms in the inventory
Inventory.prototype.vms = [];


// - --------------------------------------------
// - Must be called before using an instance of Inventory
// - once initialized, callback is invoked.
//- 
// - callback signature: function( error )
// - 
Inventory.prototype.init = function( callback ){
    
    //TODO: handle multiple calls to init.

    // ensure the inventory file exists and is 
    if( !fs.existsSync( filename ) ){
        debug( "Unable to init.  vmInventory does !exist" );
        callback( new Error( "vm inventory file does not exist.  validate your hypervisor installation." ));
        return;
    }

    // watch the inventory file
    this.watcher = chokidar.watch( filename );

    // on changes, update the inventory cache
    this.watcher.on('change', function(path){
        debug('inventory file changed');
        update_cached_inventory(true, null); 
    })

    // watch out for errors
    this.watcher.on('error', function(error){
        debug('watcher error');
        debug(error);
    })

    // load the inventory list, call the callback once loaded.
    update_cached_inventory( false, callback );

    self = this;

    // private helper to reload and update our inventory list
    function update_cached_inventory( emitIt, cb ){
        Inventory.reload_inventory( function(error, vms){
            cb = cb || function(){};
            if(error){
                if( emitIt ) self.emit('changed', error);
                cb( error );
            }else {
                self.vms = vms;
                if( emitIt ) self.emit('changed');
                cb( null );
            }
        });
    }
}


// - --------------------------------------------
// - Must be called if using mutltiple instances of this class
// - this closes the underlying file system watcher
// - 
Inventory.prototype.close = function(){
    this.watcher.close();
}


// - --------------------------------------------
// - reloads the vm inventory from disk
// - returns the inventory in the callback
//- 
// - callback: function( error, <array_of_vm_objects> )
// - 
 Inventory.reload_inventory = function( callback ){

    // cleans up quoted strings from the inventory file
    var dequote = function( key, value, section ){
        if( this.isProperty ){
            return value.replace(/\"/g, "");
        }
        return this.assert();
    }

    // options for parsing the inventory file
    var options = { path: true, namespaces: true, reviver: dequote };

    // parse it
    properties.parse( filename, options, function(error, obj){
        if( error ){
            debug("error extracting properties from inventory file");
            debug(error);
            callback( error, null );
            return;
        }

        vms = [];
        for( var vm in obj ){
            if( obj[vm].hasOwnProperty("UUID") ){ 
                vms.push(obj[vm]); 
            }
        }
        callback(null, vms);
    });
}


// export it
module.exports = Inventory;